BEGIN;
CALL insert_unit ('fl. oz. (US)');
CALL insert_unit ('Pint (US)');
CALL insert_unit ('Tbsp (US)');
CALL insert_unit ('tsp (US)');
CALL insert_unit ('Cup (US)');
CALL insert_unit ('Cup (US Legal)');
CALL insert_unit ('Quart (US)');
CALL insert_unit ('Gallon (US)');
CALL insert_unit ('tsp (metric)');
CALL insert_unit ('Tbsp (metric)');
CALL insert_unit ('Cup (metric)');
-- using customary cup as per https://en.wikipedia.org/wiki/Cup_(unit)
CALL insert_unit_conversion ('tsp (metric)', 'mL', 5);
CALL insert_unit_conversion ('Tbsp (metric)', 'mL', 15);
CALL insert_unit_conversion ('Cup (metric)', 'mL', 250);
CALL insert_unit_conversion ('Cup (US)', 'mL', 236.5882365);
CALL insert_unit_conversion ('Cup (US Legal)', 'mL', 240);
CALL insert_unit_conversion ('Cup (US)', 'Tbsp (US)', 16);
CALL insert_unit_conversion ('Cup (US)', 'tsp (US)',  48);
CALL insert_unit_conversion ('Cup (US)', 'fl. oz. (US)', 8);
CALL insert_unit_conversion ('Pint (US)', 'Cup (US)', 2);
CALL insert_unit_conversion ('Quart (US)', 'Cup (US)', 4);
CALL insert_unit_conversion ('Gallon (US)', 'Cup (US)', 16);
SELECT * FROM conversions;
DO
$$
DECLARE
  item record;
  msg text;
  expected numeric(20,10);
  result numeric(20,10);
BEGIN
  FOR item IN SELECT * FROM (VALUES
      ('L', 'mL', '', 1000), -- *
      ('mL', 'L', '', 1.0/1000),
      ('tsp (metric)', 'mL', '', 5), -- *
      ('Tbsp (metric)', 'mL', '', 15), -- *
      ('Cup (metric)', 'mL', '', 250), -- *
      ('Cup (US)', 'tsp (metric)', '', 236.5882365/5),
      ('Cup (US)', 'Tbsp (metric)', '', 236.5882365/15),
      ('Cup (US Legal)', 'mL', '', 240), -- *
      ('Cup (US Legal)', 'Tbsp (metric)', '', 16),
      ('L', 'Cup (metric)', '', 4),
      ('Cup (US)', 'Cup (metric)', '', 236.5882365/250),
      ('Cup (US)', 'mL', '', 236.5882365),
      ('mL', 'Cup (US)', '', 1/236.5882365), -- *
      ('L', 'Cup (US)', '', 1000/236.5882365),
      ('Cup (US)', 'Tbsp (US)', '', 16), -- *
      ('mL', 'Tbsp (US)', '', 16/236.5882365),
      ('Tbsp (US)', 'tsp (US)', '', 3), -- *
      ('mL', 'tsp (US)', '', 16*3/236.5882365),
      ('Pint (US)', 'fl. oz. (US)', '', 16), -- *
      ('Pint (US)', 'Cup (US)', '', 2), -- *
      ('Cup (US)', 'fl. oz. (US)', '', 8),
      ('fl. oz. (US)', 'Tbsp (US)', '', 16*2/16),
      ('fl. oz. (US)', 'tsp (US)', '', 16*2*3/16),
      ('Quart (US)', 'fl. oz. (US)', '', 32), -- *
      ('fl. oz. (US)', 'Quart (US)', '', 1.0/32),
      ('Gallon (US)', 'fl. oz. (US)', '', 128),
      ('Gallon (US)', 'Pint (US)', '', 8),
      ('Gallon (US)', 'Quart (US)', '', 4), -- *
      ('Gallon (US)', 'mL', '', 4*32*236.5882365/8)
      
    ) AS tests(f,t,p,e) LOOP
    BEGIN
      SELECT convert_unit(item.f, item.t, item.p) INTO result;
      SELECT item.e INTO expected;
      ASSERT result = expected, 'convert_unit('''||item.f||''', '''||item.t||''', '''||item.p||''') = '||result||' != '||expected;
      RAISE NOTICE ' convert_unit(''%'', ''%'', ''%'') = %', item.f, item.t, item.p, result;
      EXCEPTION
        WHEN assert_failure THEN
        BEGIN
          GET STACKED DIAGNOSTICS msg = MESSAGE_TEXT;
          RAISE WARNING '%', msg;
        END;
    END;
  END LOOP;

END;
$$ LANGUAGE plpgsql;
ROLLBACK;

